#include <stdbool.h>
#include <avr/io.h>

#include "LUFA/Drivers/USB/USB.h"                    // USB Functionality
#include "midi.h"

/** Sends a MIDI note change event (note on or off) to the MIDI output jack, on the given virtual cable ID and channel.
*
*  \param Pitch    Pitch of the note to turn on or off
*  \param OnOff    Set to true if the note is on (being held down), or false otherwise
*  \param CableID  ID of the virtual cable to send the note change to
*  \param Channel  MIDI channel number to send the note change event to
*/
void SendMIDINoteChange(const uint8_t Pitch, const bool OnOff, const uint8_t CableID, const uint8_t Channel) {
	/* Wait until endpoint ready for more data */
	while (!(Endpoint_IsReadWriteAllowed()));

	/* Check if the message should be a Note On or Note Off command */
	uint8_t Command = ((OnOff)? MIDI_COMMAND_NOTE_ON : MIDI_COMMAND_NOTE_OFF);

	/* Write the Packet Header to the endpoint */
	Endpoint_Write_Byte((CableID << 4) | (Command >> 4));

	/* Write the Note On/Off command with the specified channel, pitch and velocity */
	Endpoint_Write_Byte(Command | Channel);
	Endpoint_Write_Byte(Pitch);
	Endpoint_Write_Byte(MIDI_STANDARD_VELOCITY);

	/* Send the data in the endpoint to the host */
	Endpoint_ClearIN();
}

void SendMIDICC(const uint8_t controllerNum, const uint8_t value, const uint8_t CableID, const uint8_t Channel) {
	/* Wait until endpoint ready for more data */
	while (!(Endpoint_IsReadWriteAllowed()));

	/* Write the Packet Header to the endpoint */
	Endpoint_Write_Byte((CableID << 4) | (MIDI_COMMAND_CC >> 4));

	/* Write the CC command with controller number and value */
	Endpoint_Write_Byte(MIDI_COMMAND_CC | Channel);
	Endpoint_Write_Byte(controllerNum);
	Endpoint_Write_Byte(value);

	/* Send the data in the endpoint to the host */
	Endpoint_ClearIN();
}
